@@ -125,14 +125,18 @@ static void init_cache(
 	static bool   first_init = true;
 	static bool   running_tcp_reuseport;
 	static bool   running_socket_affinity;
+	static bool   running_xdp_tcp;
+	static bool   running_route_check;
 	static size_t running_udp_threads;
 	static size_t running_tcp_threads;
 	static size_t running_xdp_threads;
 	static size_t running_bg_threads;
 
 	if (first_init || reinit_cache) {
-		running_tcp_reuseport = conf_tcp_reuseport(conf);
-		running_socket_affinity = conf_socket_affinity(conf);
+		running_tcp_reuseport = conf_get_bool(conf, C_SRV, C_TCP_REUSEPORT);
+		running_socket_affinity = conf_get_bool(conf, C_SRV, C_SOCKET_AFFINITY);
+		running_xdp_tcp = conf_get_bool(conf, C_XDP, C_TCP);
+		running_route_check = conf_get_bool(conf, C_XDP, C_ROUTE_CHECK);
 		running_udp_threads = conf_udp_threads(conf);
 		running_tcp_threads = conf_tcp_threads(conf);
 		running_xdp_threads = conf_xdp_threads(conf);
@@ -142,27 +146,15 @@ static void init_cache(
 	}
 
 	conf_val_t val = conf_get(conf, C_SRV, C_UDP_MAX_PAYLOAD_IPV4);
-	if (val.code != KNOT_EOK) {
-		val = conf_get(conf, C_SRV, C_MAX_IPV4_UDP_PAYLOAD);
-	}
 	if (val.code != KNOT_EOK) {
 		val = conf_get(conf, C_SRV, C_UDP_MAX_PAYLOAD);
 	}
-	if (val.code != KNOT_EOK) {
-		val = conf_get(conf, C_SRV, C_MAX_UDP_PAYLOAD);
-	}
 	conf->cache.srv_udp_max_payload_ipv4 = conf_int(&val);
 
 	val = conf_get(conf, C_SRV, C_UDP_MAX_PAYLOAD_IPV6);
-	if (val.code != KNOT_EOK) {
-		val = conf_get(conf, C_SRV, C_MAX_IPV6_UDP_PAYLOAD);
-	}
 	if (val.code != KNOT_EOK) {
 		val = conf_get(conf, C_SRV, C_UDP_MAX_PAYLOAD);
 	}
-	if (val.code != KNOT_EOK) {
-		val = conf_get(conf, C_SRV, C_MAX_UDP_PAYLOAD);
-	}
 	conf->cache.srv_udp_max_payload_ipv6 = conf_int(&val);
 
 	val = conf_get(conf, C_SRV, C_TCP_IDLE_TIMEOUT);
@@ -172,16 +164,10 @@ static void init_cache(
 	conf->cache.srv_tcp_io_timeout = infinite_adjust(conf_int(&val));
 
 	val = conf_get(conf, C_SRV, C_TCP_RMT_IO_TIMEOUT);
-	if (val.code == KNOT_EOK) {
-		conf->cache.srv_tcp_remote_io_timeout = infinite_adjust(conf_int(&val));
-	} else {
-		int timeout = conf_int(&val); // New default value.
-		conf_val_t legacy = conf_get(conf, C_SRV, C_TCP_REPLY_TIMEOUT);
-		if (legacy.code == KNOT_EOK) {
-			timeout = 1000 * conf_int(&legacy); // Explicit legacy value.
-		}
-		conf->cache.srv_tcp_remote_io_timeout = infinite_adjust(timeout);
-	}
+	conf->cache.srv_tcp_remote_io_timeout = infinite_adjust(conf_int(&val));
+
+	val = conf_get(conf, C_SRV, C_TCP_FASTOPEN);
+	conf->cache.srv_tcp_fastopen = conf_bool(&val);
 
 	conf->cache.srv_tcp_reuseport = running_tcp_reuseport;
 
@@ -197,6 +183,22 @@ static void init_cache(
 
 	conf->cache.srv_tcp_max_clients = conf_tcp_max_clients(conf);
 
+	val = conf_get(conf, C_XDP, C_TCP_MAX_CLIENTS);
+	conf->cache.xdp_tcp_max_clients = conf_int(&val);
+
+	val = conf_get(conf, C_XDP, C_TCP_INBUF_MAX_SIZE);
+	conf->cache.xdp_tcp_inbuf_max_size = conf_int(&val);
+
+	val = conf_get(conf, C_XDP, C_TCP_IDLE_CLOSE);
+	conf->cache.xdp_tcp_idle_close = conf_int(&val);
+
+	val = conf_get(conf, C_XDP, C_TCP_IDLE_RESET);
+	conf->cache.xdp_tcp_idle_reset = conf_int(&val);
+
+	conf->cache.xdp_tcp = running_xdp_tcp;
+
+	conf->cache.xdp_route_check = running_route_check;
+
 	val = conf_get(conf, C_CTL, C_TIMEOUT);
 	conf->cache.ctl_timeout = conf_int(&val) * 1000;
 	/* infinite_adjust() call isn't needed, 0 is adjusted later anyway. */
