@@ -1,4 +1,4 @@
-/*  Copyright (C) 2020 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2021 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -20,7 +20,7 @@
 #include "libknot/yparser/ypschema.h"
 #include "contrib/qp-trie/trie.h"
 #include "contrib/ucw/lists.h"
-#include "contrib/dynarray.h"
+#include "libknot/dynarray.h"
 #include "knot/include/module.h"
 
 /*! Default template identifier. */
@@ -31,8 +31,16 @@
 #define CONF_DEFAULT_DBDIR	(STORAGE_DIR "/confdb")
 /*! Maximum depth of nested transactions. */
 #define CONF_MAX_TXN_DEPTH	5
+
+/*! Maximum number of UDP workers. */
+#define CONF_MAX_UDP_WORKERS	256
+/*! Maximum number of TCP workers. */
+#define CONF_MAX_TCP_WORKERS	256
+/*! Maximum number of background workers. */
+#define CONF_MAX_BG_WORKERS	512
 /*! Maximum number of concurrent DB readers. */
-#define CONF_MAX_DB_READERS	630
+#define CONF_MAX_DB_READERS	(CONF_MAX_UDP_WORKERS + CONF_MAX_TCP_WORKERS + \
+				 CONF_MAX_BG_WORKERS + 128 /* XDP workers */)
 
 /*! Configuration specific logging. */
 #define CONF_LOG(severity, msg, ...) do { \
@@ -70,8 +78,8 @@ typedef struct {
 	bool temporary;
 } module_t;
 
-dynarray_declare(mod, module_t *, DYNARRAY_VISIBILITY_PUBLIC, 16)
-dynarray_declare(old_schema, yp_item_t *, DYNARRAY_VISIBILITY_PUBLIC, 16)
+knot_dynarray_declare(mod, module_t *, DYNARRAY_VISIBILITY_NORMAL, 16)
+knot_dynarray_declare(old_schema, yp_item_t *, DYNARRAY_VISIBILITY_NORMAL, 16)
 
 struct knot_catalog;
 
@@ -114,11 +122,19 @@ typedef struct {
 		int srv_tcp_io_timeout;
 		int srv_tcp_remote_io_timeout;
 		bool srv_tcp_reuseport;
+		bool srv_tcp_fastopen;
+		bool srv_socket_affinity;
 		size_t srv_udp_threads;
 		size_t srv_tcp_threads;
 		size_t srv_xdp_threads;
 		size_t srv_bg_threads;
 		size_t srv_tcp_max_clients;
+		size_t xdp_tcp_max_clients;
+		size_t xdp_tcp_inbuf_max_size;
+		uint32_t xdp_tcp_idle_close;
+		uint32_t xdp_tcp_idle_reset;
+		bool xdp_tcp;
+		bool xdp_route_check;
 		int ctl_timeout;
 		conf_val_t srv_nsid;
 		bool srv_ecs;
