@@ -70,7 +70,11 @@ int check_listen(
 	knotd_conf_check_args_t *args
 );
 
-int check_xdp(
+int check_xdp_listen_old(
+	knotd_conf_check_args_t *args
+);
+
+int check_xdp_listen(
 	knotd_conf_check_args_t *args
 );
 
@@ -86,6 +90,10 @@ int check_server(
 	knotd_conf_check_args_t *args
 );
 
+int check_xdp(
+	knotd_conf_check_args_t *args
+);
+
 int check_keystore(
 	knotd_conf_check_args_t *args
 );
@@ -106,6 +114,10 @@ int check_remote(
 	knotd_conf_check_args_t *args
 );
 
+int check_catalog_group(
+	knotd_conf_check_args_t *args
+);
+
 int check_template(
 	knotd_conf_check_args_t *args
 );
