@@ -1,4 +1,4 @@
-/*  Copyright (C) 2020 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2021 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -28,17 +28,18 @@
 #include "knot/dnssec/zone-nsec.h"
 #include "knot/dnssec/zone-sign.h"
 #include "knot/zone/adjust.h"
+#include "knot/zone/digest.h"
 
-static int sign_init(zone_contents_t *zone, zone_sign_flags_t flags, zone_sign_roll_flags_t roll_flags,
-                     knot_time_t adjust_now, knot_lmdb_db_t *kaspdb, kdnssec_ctx_t *ctx,
+static int sign_init(zone_update_t *update, conf_t *conf, zone_sign_flags_t flags, zone_sign_roll_flags_t roll_flags,
+                     knot_time_t adjust_now, knot_lmdb_db_t *kaspdb, kdnssec_ctx_t *ctx, unsigned *zonemd_alg,
                      zone_sign_reschedule_t *reschedule)
 {
-	assert(zone);
+	assert(update);
 	assert(ctx);
 
-	const knot_dname_t *zone_name = zone->apex->owner;
+	const knot_dname_t *zone_name = update->new_cont->apex->owner;
 
-	int r = kdnssec_ctx_init(conf(), ctx, zone_name, kaspdb, NULL);
+	int r = kdnssec_ctx_init(conf, ctx, zone_name, kaspdb, NULL);
 	if (r != KNOT_EOK) {
 		return r;
 	}
@@ -56,7 +57,7 @@ static int sign_init(zone_contents_t *zone, zone_sign_flags_t flags, zone_sign_r
 	}
 
 	// update policy based on the zone content
-	update_policy_from_zone(ctx->policy, zone);
+	update_policy_from_zone(ctx->policy, update->new_cont);
 
 	// perform key rollover if needed
 	r = knot_dnssec_key_rollover(ctx, roll_flags, reschedule);
@@ -68,6 +69,15 @@ static int sign_init(zone_contents_t *zone, zone_sign_flags_t flags, zone_sign_r
 
 	ctx->rrsig_drop_existing = flags & ZONE_SIGN_DROP_SIGNATURES;
 
+	conf_val_t val = conf_zone_get(conf, C_ZONEMD_GENERATE, zone_name);
+	*zonemd_alg = conf_opt(&val);
+	if (*zonemd_alg != ZONE_DIGEST_NONE) {
+		r = zone_update_add_digest(update, *zonemd_alg, true);
+		if (r != KNOT_EOK) {
+			return r;
+		}
+	}
+
 	return KNOT_EOK;
 }
 
@@ -146,6 +156,7 @@ int knot_dnssec_nsec3resalt(kdnssec_ctx_t *ctx, knot_time_t *salt_changed, knot_
 }
 
 int knot_dnssec_zone_sign(zone_update_t *update,
+                          conf_t *conf,
                           zone_sign_flags_t flags,
                           zone_sign_roll_flags_t roll_flags,
                           knot_time_t adjust_now,
@@ -159,11 +170,12 @@ int knot_dnssec_zone_sign(zone_update_t *update,
 	const knot_dname_t *zone_name = update->new_cont->apex->owner;
 	kdnssec_ctx_t ctx = { 0 };
 	zone_keyset_t keyset = { 0 };
+	unsigned zonemd_alg;
 
 	// signing pipeline
 
-	result = sign_init(update->new_cont, flags, roll_flags, adjust_now,
-	                   update->zone->kaspdb, &ctx, reschedule);
+	result = sign_init(update, conf, flags, roll_flags, adjust_now,
+	                   update->zone->kaspdb, &ctx, &zonemd_alg, reschedule);
 	if (result != KNOT_EOK) {
 		log_zone_error(zone_name, "DNSSEC, failed to initialize (%s)",
 		               knot_strerror(result));
@@ -221,9 +233,9 @@ int knot_dnssec_zone_sign(zone_update_t *update,
 	}
 
 	if (!(flags & ZONE_SIGN_KEEP_SERIAL) && zone_update_to(update) == NULL) {
-		result = zone_update_increment_soa(update, conf());
+		result = zone_update_increment_soa(update, conf);
 		if (result == KNOT_EOK) {
-			result = knot_zone_sign_soa(update, &keyset, &ctx);
+			result = knot_zone_sign_apex_rr(update, KNOT_RRTYPE_SOA, &keyset, &ctx);
 		}
 		if (result != KNOT_EOK) {
 			log_zone_error(zone_name, "DNSSEC, failed to update SOA record (%s)",
@@ -232,6 +244,18 @@ int knot_dnssec_zone_sign(zone_update_t *update,
 		}
 	}
 
+	if (zonemd_alg != ZONE_DIGEST_NONE) {
+		result = zone_update_add_digest(update, zonemd_alg, false);
+		if (result == KNOT_EOK) {
+			result = knot_zone_sign_apex_rr(update, KNOT_RRTYPE_ZONEMD, &keyset, &ctx);
+		}
+		if (result != KNOT_EOK) {
+			log_zone_error(zone_name, "DNSSEC, failed to update ZONEMD record (%s)",
+			               knot_strerror(result));
+			goto done;
+		}
+	}
+
 	log_zone_info(zone_name, "DNSSEC, successfully signed");
 
 done:
@@ -248,7 +272,7 @@ done:
 	return result;
 }
 
-int knot_dnssec_sign_update(zone_update_t *update, zone_sign_reschedule_t *reschedule)
+int knot_dnssec_sign_update(zone_update_t *update, conf_t *conf, zone_sign_reschedule_t *reschedule)
 {
 	if (update == NULL || reschedule == NULL) {
 		return KNOT_EINVAL;
@@ -258,8 +282,9 @@ int knot_dnssec_sign_update(zone_update_t *update, zone_sign_reschedule_t *resch
 	const knot_dname_t *zone_name = update->new_cont->apex->owner;
 	kdnssec_ctx_t ctx = { 0 };
 	zone_keyset_t keyset = { 0 };
+	unsigned zonemd_alg;
 
-	result = sign_init(update->new_cont, 0, 0, 0, update->zone->kaspdb, &ctx, reschedule);
+	result = sign_init(update, conf, 0, 0, 0, update->zone->kaspdb, &ctx, &zonemd_alg, reschedule);
 	if (result != KNOT_EOK) {
 		log_zone_error(zone_name, "DNSSEC, failed to initialize (%s)",
 		               knot_strerror(result));
@@ -309,10 +334,10 @@ int knot_dnssec_sign_update(zone_update_t *update, zone_sign_reschedule_t *resch
 
 	if (!soa_changed) {
 		// incrementing SOA just of it has not been modified by the update
-		result = zone_update_increment_soa(update, conf());
+		result = zone_update_increment_soa(update, conf);
 	}
 	if (result == KNOT_EOK) {
-		result = knot_zone_sign_soa(update, &keyset, &ctx);
+		result = knot_zone_sign_apex_rr(update, KNOT_RRTYPE_SOA, &keyset, &ctx);
 	}
 	if (result != KNOT_EOK) {
 		log_zone_error(zone_name, "DNSSEC, failed to update SOA record (%s)",
@@ -320,6 +345,18 @@ int knot_dnssec_sign_update(zone_update_t *update, zone_sign_reschedule_t *resch
 		goto done;
 	}
 
+	if (zonemd_alg != ZONE_DIGEST_NONE) {
+		result = zone_update_add_digest(update, zonemd_alg, false);
+		if (result == KNOT_EOK) {
+			result = knot_zone_sign_apex_rr(update, KNOT_RRTYPE_ZONEMD, &keyset, &ctx);
+		}
+		if (result != KNOT_EOK) {
+			log_zone_error(zone_name, "DNSSEC, failed to update ZONEMD record (%s)",
+			               knot_strerror(result));
+			goto done;
+		}
+	}
+
 	log_zone_info(zone_name, "DNSSEC, successfully signed");
 
 done:
@@ -342,10 +379,10 @@ knot_time_t knot_dnssec_failover_delay(const kdnssec_ctx_t *ctx)
 	}
 }
 
-int knot_dnssec_validate_zone(zone_update_t *update, bool incremental)
+int knot_dnssec_validate_zone(zone_update_t *update, conf_t *conf, bool incremental)
 {
 	kdnssec_ctx_t ctx = { 0 };
-	int ret = kdnssec_validation_ctx(conf(), &ctx, update->new_cont);
+	int ret = kdnssec_validation_ctx(conf, &ctx, update->new_cont);
 	if (ret == KNOT_EOK) {
 		ret = knot_zone_check_nsec_chain(update, &ctx, incremental);
 	}
