@@ -1,4 +1,4 @@
-/*  Copyright (C) 2020 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2021 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -28,8 +28,7 @@
 #include "knot/dnssec/rrset-sign.h"
 #include "knot/dnssec/zone-sign.h"
 #include "libknot/libknot.h"
-#include "contrib/dynarray.h"
-#include "contrib/macros.h"
+#include "libknot/dynarray.h"
 #include "contrib/wire_ctx.h"
 
 typedef struct {
@@ -573,9 +572,8 @@ static void *tree_sign_thread(void *_arg)
 	return NULL;
 }
 
-static int set_signed(zone_node_t *node, void *data)
+static int set_signed(zone_node_t *node, _unused_ void *data)
 {
-	UNUSED(data);
 	node->flags |= NODE_FLAGS_RRSIGS_VALID;
 	return KNOT_EOK;
 }
@@ -762,6 +760,7 @@ keyptr_dynarray_t knot_zone_sign_get_cdnskeys(const kdnssec_ctx_t *ctx,
 {
 	keyptr_dynarray_t r = { 0 };
 	unsigned crp = ctx->policy->cds_cdnskey_publish;
+	unsigned cds_published = 0;
 
 	if (crp == CDS_CDNSKEY_ROLLOVER || crp == CDS_CDNSKEY_ALWAYS ||
 	    crp == CDS_CDNSKEY_DOUBLE_DS) {
@@ -771,11 +770,14 @@ keyptr_dynarray_t knot_zone_sign_get_cdnskeys(const kdnssec_ctx_t *ctx,
 			if (key->is_ready) {
 				assert(key->is_ksk);
 				keyptr_dynarray_add(&r, &key);
+				if (!key->is_pub_only) {
+					cds_published++;
+				}
 			}
 		}
 
 		// second, add active keys
-		if ((crp == CDS_CDNSKEY_ALWAYS && r.size == 0) ||
+		if ((crp == CDS_CDNSKEY_ALWAYS && cds_published == 0) ||
 		    (crp == CDS_CDNSKEY_DOUBLE_DS)) {
 			for (int i = 0; i < zone_keys->count; i++) {
 				zone_key_t *key = &zone_keys->keys[i];
@@ -785,9 +787,9 @@ keyptr_dynarray_t knot_zone_sign_get_cdnskeys(const kdnssec_ctx_t *ctx,
 			}
 		}
 
-		if ((crp != CDS_CDNSKEY_DOUBLE_DS && r.size > 1) ||
-		    (r.size > 2)) {
-			log_zone_warning(ctx->zone->dname, "DNSSEC, published CDS/CDNSKEY records for too many (%zu) keys", r.size);
+		if ((crp != CDS_CDNSKEY_DOUBLE_DS && cds_published > 1) ||
+		    (cds_published > 2)) {
+			log_zone_warning(ctx->zone->dname, "DNSSEC, published CDS/CDNSKEY records for too many (%u) keys", cds_published);
 		}
 	}
 
@@ -811,7 +813,7 @@ int knot_zone_sign_add_dnskeys(zone_keyset_t *zone_keys, const kdnssec_ctx_t *dn
 		}
 	}
 	keyptr_dynarray_t kcdnskeys = knot_zone_sign_get_cdnskeys(dnssec_ctx, zone_keys);
-	dynarray_foreach(keyptr, zone_key_t *, ksk_for_cds, kcdnskeys) {
+	knot_dynarray_foreach(keyptr, zone_key_t *, ksk_for_cds, kcdnskeys) {
 		ret = rrset_add_zone_key(&add_r->cdnskey, *ksk_for_cds);
 		if (ret == KNOT_EOK) {
 			ret = rrset_add_zone_ds(&add_r->cds, *ksk_for_cds, dnssec_ctx->policy->cds_dt);
@@ -1073,12 +1075,12 @@ int knot_zone_sign_update(zone_update_t *update,
 	return ret;
 }
 
-int knot_zone_sign_soa(zone_update_t *update,
-		       const zone_keyset_t *zone_keys,
-		       const kdnssec_ctx_t *dnssec_ctx)
+int knot_zone_sign_apex_rr(zone_update_t *update, uint16_t rrtype,
+                           const zone_keyset_t *zone_keys,
+                           const kdnssec_ctx_t *dnssec_ctx)
 {
-	knot_rrset_t soa_to = node_rrset(update->new_cont->apex, KNOT_RRTYPE_SOA);
-	knot_rrset_t soa_rrsig = node_rrset(update->new_cont->apex, KNOT_RRTYPE_RRSIG);
+	knot_rrset_t rr = node_rrset(update->new_cont->apex, rrtype);
+	knot_rrset_t rrsig = node_rrset(update->new_cont->apex, KNOT_RRTYPE_RRSIG);
 	changeset_t ch;
 	int ret = changeset_init(&ch, update->zone->name);
 	if (ret == KNOT_EOK) {
@@ -1087,7 +1089,7 @@ int knot_zone_sign_soa(zone_update_t *update,
 			changeset_clear(&ch);
 			return KNOT_ENOMEM;
 		}
-		ret = force_resign_rrset(&soa_to, &soa_rrsig, sign_ctx, &ch);
+		ret = force_resign_rrset(&rr, &rrsig, sign_ctx, &ch);
 		if (ret == KNOT_EOK) {
 			ret = zone_update_apply_changeset(update, &ch);
 		}
