@@ -1,4 +1,4 @@
-/*  Copyright (C) 2020 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2021 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -53,6 +53,7 @@ typedef struct zone_events {
 
 	zone_event_type_t type;		//!< Type of running event.
 	bool running;			//!< Some zone event is being run.
+	pthread_cond_t *run_end;	//!< Notify this one after finishing a job.
 
 	bool frozen;			//!< Terminated, don't schedule new events.
 	bool ufrozen;			//!< Updates to the zone temporarily frozen by user.
@@ -64,6 +65,7 @@ typedef struct zone_events {
 	time_t time[ZONE_EVENT_COUNT];	//!< Event execution times.
 	bool forced[ZONE_EVENT_COUNT];  //!< Flag that the event was invoked by user ctl.
 	pthread_cond_t *blocking[ZONE_EVENT_COUNT];       //!< For blocking events: dispatching cond.
+	int result[ZONE_EVENT_COUNT];   //!< Event return values (in blocking operations).
 } zone_events_t;
 
 /*!
@@ -147,8 +149,10 @@ void zone_events_schedule_user(struct zone *zone, zone_event_type_t type);
  * \param zone  Zone to schedule new event for.
  * \param type  Zone event type.
  * \param user  Forced flag indication.
+ *
+ * \return KNOT_E*
  */
-void zone_events_schedule_blocking(struct zone *zone, zone_event_type_t type, bool user);
+int zone_events_schedule_blocking(struct zone *zone, zone_event_type_t type, bool user);
 
 /*!
  * \brief Freeze all zone events and prevent new events from running.
@@ -157,6 +161,13 @@ void zone_events_schedule_blocking(struct zone *zone, zone_event_type_t type, bo
  */
 void zone_events_freeze(struct zone *zone);
 
+/*!
+ * \brief Freeze zone events and wait for running event to finish.
+ *
+ * \param zone  Zone to freeze events for.
+ */
+void zone_events_freeze_blocking(struct zone *zone);
+
 /*!
  * \brief ufreeze_applies
  * \param type Type of event to be checked
