@@ -1,4 +1,4 @@
-/*  Copyright (C) 2020 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2021 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -153,27 +153,6 @@ static time_t bootstrap_next(const zone_timers_t *timers)
 	return interval;
 }
 
-static int xfr_validate(zone_contents_t *zone, zone_tree_t *update_node_ptrs)
-{
-	// adjust_cb_nsec3_pointer not needed as we don't check DNSSEC here
-	int ret = zone_adjust_contents(zone, adjust_cb_flags, NULL, false, false, 1, update_node_ptrs);
-	if (ret != KNOT_EOK) {
-		return ret;
-	}
-
-	sem_handler_t handler = {
-		.cb = err_handler_logger
-	};
-
-	ret = sem_checks_process(zone, SEMCHECK_MANDATORY_ONLY, &handler, time(NULL));
-	if (ret != KNOT_EOK) {
-		// error is logged by the error handler
-		return ret;
-	}
-
-	return KNOT_EOK;
-}
-
 static void xfr_log_publish(const struct refresh_data *data,
                             const uint32_t old_serial,
                             const uint32_t new_serial,
@@ -235,16 +214,13 @@ static void axfr_slave_sign_serial(zone_contents_t *new_contents, zone_t *zone,
 	uint32_t new_serial, lastsigned_serial;
 	if (zone->contents != NULL) {
 		// Retransfer or AXFR-fallback - increment current serial.
-		new_serial = serial_next(zone_contents_serial(zone->contents), serial_policy);
+		new_serial = serial_next(zone_contents_serial(zone->contents), serial_policy, 1);
 	} else if (zone_get_lastsigned_serial(zone, &lastsigned_serial) == KNOT_EOK) {
 		// Bootstrap - increment stored serial.
-		new_serial = serial_next(lastsigned_serial, serial_policy);
-	} else if (serial_must_increment(*master_serial, serial_policy)) {
-		// Bootstrap - increment master's serial, consider policy.
-		new_serial = serial_next(*master_serial, serial_policy);
+		new_serial = serial_next(lastsigned_serial, serial_policy, 1);
 	} else {
-		// Bootstrap - simply use master's serial.
-		new_serial = *master_serial;
+		// Bootstrap - try to reuse master's serial, considering policy.
+		new_serial = serial_next(*master_serial, serial_policy, 0);
 	}
 	zone_contents_set_soa_serial(new_contents, new_serial);
 }
@@ -253,11 +229,6 @@ static int axfr_finalize(struct refresh_data *data)
 {
 	zone_contents_t *new_zone = data->axfr.zone;
 
-	int ret = xfr_validate(new_zone, NULL);
-	if (ret != KNOT_EOK) {
-		return ret;
-	}
-
 	conf_val_t val = conf_zone_get(data->conf, C_DNSSEC_SIGNING, data->zone->name);
 	bool dnssec_enable = conf_bool(&val);
 	uint32_t old_serial = zone_contents_serial(data->zone->contents), master_serial = 0;
@@ -268,17 +239,25 @@ static int axfr_finalize(struct refresh_data *data)
 	}
 
 	zone_update_t up = { 0 };
-	ret = zone_update_from_contents(&up, data->zone, new_zone, UPDATE_FULL);
+	int ret = zone_update_from_contents(&up, data->zone, new_zone, UPDATE_FULL);
 	if (ret != KNOT_EOK) {
 		return ret;
 	}
-
 	// Seized by zone_update. Don't free the contents again in axfr_cleanup.
 	data->axfr.zone = NULL;
 
+	ret = zone_update_semcheck(&up);
+	if (ret == KNOT_EOK) {
+		ret = zone_update_verify_digest(data->conf, &up);
+	}
+	if (ret != KNOT_EOK) {
+		zone_update_clear(&up);
+		return ret;
+	}
+
 	if (dnssec_enable) {
 		zone_sign_reschedule_t resch = { 0 };
-		ret = knot_dnssec_zone_sign(&up, ZONE_SIGN_KEEP_SERIAL, KEY_ROLL_ALLOW_ALL, 0, &resch);
+		ret = knot_dnssec_zone_sign(&up, data->conf, ZONE_SIGN_KEEP_SERIAL, KEY_ROLL_ALLOW_ALL, 0, &resch);
 		if (ret != KNOT_EOK) {
 			zone_update_clear(&up);
 			return ret;
@@ -452,7 +431,7 @@ static bool ixfr_serial_once(changeset_t *ch, int policy, uint32_t *master_seria
 	}
 
 	uint32_t new_from = *local_serial;
-	uint32_t new_to = serial_next(new_from, policy);
+	uint32_t new_to = serial_next(new_from, policy, 1);
 	knot_soa_serial_set(ch->soa_from->rrs.rdata, new_from);
 	knot_soa_serial_set(ch->soa_to->rrs.rdata, new_to);
 
@@ -527,7 +506,10 @@ static int ixfr_finalize(struct refresh_data *data)
 		}
 	}
 
-	ret = xfr_validate(up.new_cont, up.a_ctx->node_ptrs);
+	ret = zone_update_semcheck(&up);
+	if (ret == KNOT_EOK) {
+		ret = zone_update_verify_digest(data->conf, &up);
+	}
 	if (ret != KNOT_EOK) {
 		zone_update_clear(&up);
 		return ret;
@@ -535,7 +517,7 @@ static int ixfr_finalize(struct refresh_data *data)
 
 	if (dnssec_enable) {
 		zone_sign_reschedule_t resch = { 0 };
-		ret = knot_dnssec_sign_update(&up, &resch);
+		ret = knot_dnssec_sign_update(&up, data->conf, &resch);
 		if (ret != KNOT_EOK) {
 			zone_update_clear(&up);
 			return ret;
@@ -1106,9 +1088,6 @@ static const knot_layer_api_t REFRESH_API = {
 static size_t max_zone_size(conf_t *conf, const knot_dname_t *zone)
 {
 	conf_val_t val = conf_zone_get(conf, C_ZONE_MAX_SIZE, zone);
-	if (val.code != KNOT_EOK) {
-		val = conf_zone_get(conf, C_MAX_ZONE_SIZE, zone);
-	}
 	return conf_int(&val);
 }
 
@@ -1199,7 +1178,8 @@ static int try_refresh(conf_t *conf, zone_t *zone, const conf_remote_t *master,
 
 	const struct sockaddr_storage *dst = &master->addr;
 	const struct sockaddr_storage *src = &master->via;
-	knot_request_t *req = knot_request_make(NULL, dst, src, pkt, &master->key, 0);
+	knot_request_flag_t flags = conf->cache.srv_tcp_fastopen ? KNOT_REQUEST_TFO : 0;
+	knot_request_t *req = knot_request_make(NULL, dst, src, pkt, &master->key, flags);
 	if (!req) {
 		knot_request_free(req, NULL);
 		knot_requestor_clear(&requestor);
@@ -1237,18 +1217,12 @@ static int try_refresh(conf_t *conf, zone_t *zone, const conf_remote_t *master,
 static int64_t min_refresh_interval(conf_t *conf, const knot_dname_t *zone)
 {
 	conf_val_t val = conf_zone_get(conf, C_REFRESH_MIN_INTERVAL, zone);
-	if (val.code != KNOT_EOK) {
-		val = conf_zone_get(conf, C_MIN_REFRESH_INTERVAL, zone);
-	}
 	return conf_int(&val);
 }
 
 static int64_t max_refresh_interval(conf_t *conf, const knot_dname_t *zone)
 {
 	conf_val_t val = conf_zone_get(conf, C_REFRESH_MAX_INTERVAL, zone);
-	if (val.code != KNOT_EOK) {
-		val = conf_zone_get(conf, C_MAX_REFRESH_INTERVAL, zone);
-	}
 	return conf_int(&val);
 }
 
@@ -1257,7 +1231,7 @@ int event_refresh(conf_t *conf, zone_t *zone)
 	assert(zone);
 
 	if (!zone_is_slave(conf, zone)) {
-		return KNOT_EOK;
+		return KNOT_ENOTSUP;
 	}
 
 	try_refresh_ctx_t trctx = { 0 };
@@ -1301,11 +1275,11 @@ int event_refresh(conf_t *conf, zone_t *zone)
 		zone->timers.next_refresh = now + max_refresh;
 	}
 
-	/* Rechedule events. */
+	/* Reschedule events. */
 	replan_from_timers(conf, zone);
 	if (trctx.send_notify) {
 		zone_events_schedule_at(zone, ZONE_EVENT_NOTIFY, time(NULL) + 1);
 	}
 
-	return KNOT_EOK;
+	return ret;
 }
