@@ -23,8 +23,10 @@
 
 #include "libknot/libknot.h"
 #include "libknot/yparser/ypschema.h"
+#include "libknot/xdp.h"
 #include "knot/common/log.h"
 #include "knot/common/stats.h"
+#include "knot/common/systemd.h"
 #include "knot/conf/confio.h"
 #include "knot/conf/migration.h"
 #include "knot/conf/module.h"
@@ -37,6 +39,7 @@
 #include "knot/zone/zonedb-load.h"
 #include "knot/worker/pool.h"
 #include "contrib/net.h"
+#include "contrib/openbsd/strlcat.h"
 #include "contrib/sockaddr.h"
 #include "contrib/trim.h"
 
@@ -218,25 +221,8 @@ static int disable_pmtudisc(int sock, int family)
 	return KNOT_EOK;
 }
 
-/*!
- * \brief Enable TCP Fast Open.
- */
-static int enable_fastopen(int sock, int backlog)
-{
-#if defined(TCP_FASTOPEN)
-#if __APPLE__
-	if (backlog > 0) {
-		backlog = 1; // just on-off switch on macOS
-	}
-#endif
-	if (setsockopt(sock, IPPROTO_TCP, TCP_FASTOPEN, &backlog, sizeof(backlog)) != 0) {
-		return knot_map_errno();
-	}
-#endif
-	return KNOT_EOK;
-}
-
-static iface_t *server_init_xdp_iface(struct sockaddr_storage *addr, unsigned *thread_id_start)
+static iface_t *server_init_xdp_iface(struct sockaddr_storage *addr, bool route_check,
+                                      bool tcp, unsigned *thread_id_start)
 {
 #ifndef ENABLE_XDP
 	assert(0);
@@ -267,16 +253,21 @@ static iface_t *server_init_xdp_iface(struct sockaddr_storage *addr, unsigned *t
 	new_if->xdp_first_thread_id = *thread_id_start;
 	*thread_id_start += iface.queues;
 
+	uint32_t xdp_flags = route_check ? KNOT_XDP_LISTEN_PORT_ROUTE : 0;
+	if (tcp) {
+		xdp_flags |= KNOT_XDP_LISTEN_PORT_TCP;
+	}
+
 	for (int i = 0; i < iface.queues; i++) {
 		knot_xdp_load_bpf_t mode =
 			(i == 0 ? KNOT_XDP_LOAD_BPF_ALWAYS : KNOT_XDP_LOAD_BPF_NEVER);
 		ret = knot_xdp_init(new_if->xdp_sockets + i, iface.name, i,
-		                    iface.port, mode);
+		                    iface.port | xdp_flags, mode);
 		if (ret == -EBUSY && i == 0) {
-			log_notice("XDP interface %s@%u is busy, retrying initializaion",
+			log_notice("XDP interface %s@%u is busy, retrying initialization",
 			           iface.name, iface.port);
 			ret = knot_xdp_init(new_if->xdp_sockets + i, iface.name, i,
-			                    iface.port, KNOT_XDP_LOAD_BPF_ALWAYS_UNLOAD);
+			                    iface.port | xdp_flags, KNOT_XDP_LOAD_BPF_ALWAYS_UNLOAD);
 		}
 		if (ret != KNOT_EOK) {
 			log_warning("failed to initialize XDP interface %s@%u, queue %d (%s)",
@@ -291,9 +282,10 @@ static iface_t *server_init_xdp_iface(struct sockaddr_storage *addr, unsigned *t
 
 	if (ret == KNOT_EOK) {
 		knot_xdp_mode_t mode = knot_eth_xdp_mode(if_nametoindex(iface.name));
-		log_debug("initialized XDP interface %s@%u, queues %d, %s mode",
-		          iface.name, iface.port, iface.queues,
-		          (mode == KNOT_XDP_MODE_FULL ? "native" : "emulated"));
+		log_debug("initialized XDP interface %s@%u UDP%s, queues %d, %s mode%s",
+		          iface.name, iface.port, (tcp ? "/TCP" : ""), iface.queues,
+		          (mode == KNOT_XDP_MODE_FULL ? "native" : "emulated"),
+		          route_check ? ", route check" : "");
 	}
 
 	return new_if;
@@ -457,9 +449,9 @@ static iface_t *server_init_iface(struct sockaddr_storage *addr,
 			}
 		}
 
-		/* TCP Fast Open. */
-		ret = enable_fastopen(sock, TCP_BACKLOG_SIZE);
-		if (ret < 0 && warn_flag_misc) {
+		/* Try to enable TCP Fast Open. */
+		ret = net_bound_tfo(sock, TCP_BACKLOG_SIZE);
+		if (ret != KNOT_EOK && ret != KNOT_ENOTSUP && warn_flag_misc) {
 			log_warning("failed to enable TCP Fast Open on %s (%s)",
 			            addr_str, knot_strerror(ret));
 			warn_flag_misc = false;
@@ -469,6 +461,34 @@ static iface_t *server_init_iface(struct sockaddr_storage *addr,
 	return new_if;
 }
 
+static void log_sock_conf(conf_t *conf)
+{
+	char buf[128] = "";
+#if defined(ENABLE_REUSEPORT)
+	strlcat(buf, "UDP", sizeof(buf));
+	if (conf->cache.srv_tcp_reuseport) {
+		strlcat(buf, "/TCP", sizeof(buf));
+	}
+	strlcat(buf, " reuseport", sizeof(buf));
+	if (conf->cache.srv_socket_affinity) {
+		strlcat(buf, ", socket affinity", sizeof(buf));
+	}
+#endif
+#if defined(TCP_FASTOPEN)
+	if (buf[0] != '\0') {
+		strlcat(buf, ", ", sizeof(buf));
+	}
+	strlcat(buf, "incoming", sizeof(buf));
+	if (conf->cache.srv_tcp_fastopen) {
+		strlcat(buf, "/outgoing", sizeof(buf));
+	}
+	strlcat(buf, " TCP Fast Open", sizeof(buf));
+#endif
+	if (buf[0] != '\0') {
+		log_info("using %s", buf);
+	}
+}
+
 /*! \brief Initialize bound sockets according to configuration. */
 static int configure_sockets(conf_t *conf, server_t *s)
 {
@@ -477,30 +497,35 @@ static int configure_sockets(conf_t *conf, server_t *s)
 	}
 
 	conf_val_t listen_val = conf_get(conf, C_SRV, C_LISTEN);
-	conf_val_t lisxdp_val = conf_get(conf, C_SRV, C_LISTEN_XDP);
+	conf_val_t lisxdp_val = conf_get(conf, C_XDP, C_LISTEN);
+	if (lisxdp_val.code != KNOT_EOK) {
+		lisxdp_val = conf_get(conf, C_SRV, C_LISTEN_XDP);
+	}
 	conf_val_t rundir_val = conf_get(conf, C_SRV, C_RUNDIR);
 
 	if (listen_val.code == KNOT_EOK) {
-#ifdef ENABLE_REUSEPORT
-		/* Log info if reuseport is used and for what protocols. */
-		log_info("using reuseport%s for UDP%s",
-			 conf->cache.srv_socket_affinity ? " with socket affinity" : "",
-		         conf->cache.srv_tcp_reuseport ? " and TCP" : "");
-#endif
+		log_sock_conf(conf);
 	} else if (lisxdp_val.code != KNOT_EOK) {
 		log_warning("no network interface configured");
 		return KNOT_EOK;
 	}
 
+#ifdef ENABLE_XDP
 	if (lisxdp_val.code == KNOT_EOK) {
-		struct rlimit no_limit = { RLIM_INFINITY, RLIM_INFINITY };
-		int ret = setrlimit(RLIMIT_MEMLOCK, &no_limit);
-		if (ret != 0) {
-			log_error("failed to increase RLIMIT_MEMLOCK (%s)",
-			          knot_strerror(errno));
-			return KNOT_ESYSTEM;
+		struct rlimit min_limit = { RLIM_INFINITY, RLIM_INFINITY };
+		struct rlimit cur_limit = { 0 };
+		if (getrlimit(RLIMIT_MEMLOCK, &cur_limit) != 0 ||
+		    cur_limit.rlim_cur != min_limit.rlim_cur ||
+		    cur_limit.rlim_max != min_limit.rlim_max) {
+			int ret = setrlimit(RLIMIT_MEMLOCK, &min_limit);
+			if (ret != 0) {
+				log_error("failed to increase RLIMIT_MEMLOCK (%s)",
+				          knot_strerror(errno));
+				return KNOT_ESYSTEM;
+			}
 		}
 	}
+#endif
 
 	size_t real_nifs = 0;
 	size_t nifs = conf_val_count(&listen_val) + conf_val_count(&lisxdp_val);
@@ -537,6 +562,8 @@ static int configure_sockets(conf_t *conf, server_t *s)
 	free(rundir);
 
 	/* XDP sockets. */
+	bool xdp_tcp = conf->cache.xdp_tcp;
+	bool route_check = conf->cache.xdp_route_check;
 	unsigned thread_id = s->handlers[IO_UDP].handler.unit->size +
 	                     s->handlers[IO_TCP].handler.unit->size;
 	while (lisxdp_val.code == KNOT_EOK) {
@@ -545,7 +572,7 @@ static int configure_sockets(conf_t *conf, server_t *s)
 		sockaddr_tostr(addr_str, sizeof(addr_str), &addr);
 		log_info("binding to XDP interface %s", addr_str);
 
-		iface_t *new_if = server_init_xdp_iface(&addr, &thread_id);
+		iface_t *new_if = server_init_xdp_iface(&addr, route_check, xdp_tcp, &thread_id);
 		if (new_if == NULL) {
 			server_deinit_iface_list(newlist, nifs);
 			return KNOT_ERROR;
@@ -601,22 +628,24 @@ int server_init(server_t *server, int bg_workers)
 		return ret;
 	}
 
+	zone_backups_init(&server->backup_ctxs);
+
 	char *catalog_dir = conf_db(conf(), C_CATALOG_DB);
-	conf_val_t catalog_size = conf_db_param(conf(), C_CATALOG_DB_MAX_SIZE, NULL);
+	conf_val_t catalog_size = conf_db_param(conf(), C_CATALOG_DB_MAX_SIZE);
 	catalog_init(&server->catalog, catalog_dir, conf_int(&catalog_size));
 	free(catalog_dir);
 	conf()->catalog = &server->catalog;
 
 	char *journal_dir = conf_db(conf(), C_JOURNAL_DB);
-	conf_val_t journal_size = conf_db_param(conf(), C_JOURNAL_DB_MAX_SIZE, C_MAX_JOURNAL_DB_SIZE);
-	conf_val_t journal_mode = conf_db_param(conf(), C_JOURNAL_DB_MODE, C_JOURNAL_DB_MODE);
+	conf_val_t journal_size = conf_db_param(conf(), C_JOURNAL_DB_MAX_SIZE);
+	conf_val_t journal_mode = conf_db_param(conf(), C_JOURNAL_DB_MODE);
 	knot_lmdb_init(&server->journaldb, journal_dir, conf_int(&journal_size), journal_env_flags(conf_opt(&journal_mode), false), NULL);
 	free(journal_dir);
 
 	kasp_db_ensure_init(&server->kaspdb, conf());
 
 	char *timer_dir = conf_db(conf(), C_TIMER_DB);
-	conf_val_t timer_size = conf_db_param(conf(), C_TIMER_DB_MAX_SIZE, C_MAX_TIMER_DB_SIZE);
+	conf_val_t timer_size = conf_db_param(conf(), C_TIMER_DB_MAX_SIZE);
 	knot_lmdb_init(&server->timerdb, timer_dir, conf_int(&timer_size), 0, NULL);
 	free(timer_dir);
 
@@ -629,6 +658,8 @@ void server_deinit(server_t *server)
 		return;
 	}
 
+	zone_backups_deinit(&server->backup_ctxs);
+
 	/* Save zone timers. */
 	if (server->zone_db != NULL) {
 		log_info("updating persistent timer DB");
@@ -652,6 +683,7 @@ void server_deinit(server_t *server)
 	evsched_deinit(&server->sched);
 
 	/* Free catalog zone context. */
+	catalog_update_clear(&server->catalog_upd);
 	catalog_update_deinit(&server->catalog_upd);
 	catalog_deinit(&server->catalog);
 
@@ -711,6 +743,22 @@ static void server_free_handler(iohandler_t *h)
 	free(h->thread_id);
 }
 
+static void worker_wait_cb(worker_pool_t *pool)
+{
+	systemd_zone_load_timeout_notify();
+
+	static uint64_t last_ns = 0;
+	struct timespec now = time_now();
+	uint64_t now_ns = 1000000000 * now.tv_sec + now.tv_nsec;
+	/* Too frequent worker_pool_status() call with many zones is expensive. */
+	if (now_ns - last_ns > 1000000000) {
+		int running, queued;
+		worker_pool_status(pool, true, &running, &queued);
+		systemd_tasks_status_notify(running + queued);
+		last_ns = now_ns;
+	}
+}
+
 int server_start(server_t *server, bool async)
 {
 	if (server == NULL) {
@@ -722,7 +770,8 @@ int server_start(server_t *server, bool async)
 
 	/* Wait for enqueued events if not asynchronous. */
 	if (!async) {
-		worker_pool_wait(server->workers);
+		worker_pool_wait_cb(server->workers, worker_wait_cb);
+		systemd_tasks_status_notify(0);
 	}
 
 	/* Start evsched handler. */
@@ -817,7 +866,10 @@ static bool listen_changed(conf_t *conf, server_t *server)
 	assert(server->ifaces);
 
 	conf_val_t listen_val = conf_get(conf, C_SRV, C_LISTEN);
-	conf_val_t lisxdp_val = conf_get(conf, C_SRV, C_LISTEN_XDP);
+	conf_val_t lisxdp_val = conf_get(conf, C_XDP, C_LISTEN);
+	if (lisxdp_val.code != KNOT_EOK) {
+		lisxdp_val = conf_get(conf, C_SRV, C_LISTEN_XDP);
+	}
 	size_t new_count = conf_val_count(&listen_val) + conf_val_count(&lisxdp_val);
 	size_t old_count = server->n_ifaces;
 	if (new_count != old_count) {
@@ -876,13 +928,15 @@ static void warn_server_reconfigure(conf_t *conf, server_t *server)
 	static bool warn_tcp = true;
 	static bool warn_bg = true;
 	static bool warn_listen = true;
+	static bool warn_xdp_tcp = true;
+	static bool warn_route_check = true;
 
-	if (warn_tcp_reuseport && conf->cache.srv_tcp_reuseport != conf_tcp_reuseport(conf)) {
+	if (warn_tcp_reuseport && conf->cache.srv_tcp_reuseport != conf_get_bool(conf, C_SRV, C_TCP_REUSEPORT)) {
 		log_warning(msg, &C_TCP_REUSEPORT[1]);
 		warn_tcp_reuseport = false;
 	}
 
-	if (warn_socket_affinity && conf->cache.srv_socket_affinity != conf_socket_affinity(conf)) {
+	if (warn_socket_affinity && conf->cache.srv_socket_affinity != conf_get_bool(conf, C_SRV, C_SOCKET_AFFINITY)) {
 		log_warning(msg, &C_SOCKET_AFFINITY[1]);
 		warn_socket_affinity = false;
 	}
@@ -906,6 +960,16 @@ static void warn_server_reconfigure(conf_t *conf, server_t *server)
 		log_warning(msg, "listen(-xdp)");
 		warn_listen = false;
 	}
+
+	if (warn_xdp_tcp && conf->cache.xdp_tcp != conf_get_bool(conf, C_XDP, C_TCP)) {
+		log_warning(msg, &C_TCP[1]);
+		warn_xdp_tcp = false;
+	}
+
+	if (warn_route_check && conf->cache.xdp_route_check != conf_get_bool(conf, C_XDP, C_ROUTE_CHECK)) {
+		log_warning(msg, &C_ROUTE_CHECK[1]);
+		warn_route_check = false;
+	}
 }
 
 int server_reload(server_t *server)
@@ -914,9 +978,12 @@ int server_reload(server_t *server)
 		return KNOT_EINVAL;
 	}
 
+	systemd_reloading_notify();
+
 	/* Check for no edit mode. */
 	if (conf()->io.txn != NULL) {
 		log_warning("reload aborted due to active configuration transaction");
+		systemd_ready_notify();
 		return KNOT_TXN_EEXISTS;
 	}
 
@@ -925,6 +992,7 @@ int server_reload(server_t *server)
 	if (ret != KNOT_EOK) {
 		log_error("failed to initialize configuration (%s)",
 		          knot_strerror(ret));
+		systemd_ready_notify();
 		return ret;
 	}
 
@@ -937,6 +1005,7 @@ int server_reload(server_t *server)
 		ret = reload_conf(new_conf);
 		if (ret != KNOT_EOK) {
 			conf_free(new_conf);
+			systemd_ready_notify();
 			return ret;
 		}
 
@@ -981,12 +1050,15 @@ int server_reload(server_t *server)
 		}
 	}
 
+	systemd_ready_notify();
+
 	return KNOT_EOK;
 }
 
 void server_stop(server_t *server)
 {
 	log_info("stopping server");
+	systemd_stopping_notify();
 
 	/* Stop scheduler. */
 	evsched_stop(&server->sched);
@@ -1030,8 +1102,8 @@ static int configure_threads(conf_t *conf, server_t *server)
 static int reconfigure_journal_db(conf_t *conf, server_t *server)
 {
 	char *journal_dir = conf_db(conf, C_JOURNAL_DB);
-	conf_val_t journal_size = conf_db_param(conf, C_JOURNAL_DB_MAX_SIZE, C_MAX_JOURNAL_DB_SIZE);
-	conf_val_t journal_mode = conf_db_param(conf, C_JOURNAL_DB_MODE, C_JOURNAL_DB_MODE);
+	conf_val_t journal_size = conf_db_param(conf, C_JOURNAL_DB_MAX_SIZE);
+	conf_val_t journal_mode = conf_db_param(conf, C_JOURNAL_DB_MODE);
 	int ret = knot_lmdb_reinit(&server->journaldb, journal_dir, conf_int(&journal_size),
 	                           journal_env_flags(conf_opt(&journal_mode), false));
 	if (ret != KNOT_EOK) {
@@ -1045,7 +1117,7 @@ static int reconfigure_journal_db(conf_t *conf, server_t *server)
 static int reconfigure_kasp_db(conf_t *conf, server_t *server)
 {
 	char *kasp_dir = conf_db(conf, C_KASP_DB);
-	conf_val_t kasp_size = conf_db_param(conf, C_KASP_DB_MAX_SIZE, C_MAX_KASP_DB_SIZE);
+	conf_val_t kasp_size = conf_db_param(conf, C_KASP_DB_MAX_SIZE);
 	int ret = knot_lmdb_reinit(&server->kaspdb, kasp_dir, conf_int(&kasp_size), 0);
 	if (ret != KNOT_EOK) {
 		log_warning("ignored reconfiguration of KASP DB (%s)", knot_strerror(ret));
@@ -1058,7 +1130,7 @@ static int reconfigure_kasp_db(conf_t *conf, server_t *server)
 static int reconfigure_timer_db(conf_t *conf, server_t *server)
 {
 	char *timer_dir = conf_db(conf, C_TIMER_DB);
-	conf_val_t timer_size = conf_db_param(conf, C_TIMER_DB_MAX_SIZE, C_MAX_TIMER_DB_SIZE);
+	conf_val_t timer_size = conf_db_param(conf, C_TIMER_DB_MAX_SIZE);
 	int ret = knot_lmdb_reconfigure(&server->timerdb, timer_dir, conf_int(&timer_size), 0);
 	free(timer_dir);
 	return ret;
@@ -1095,6 +1167,10 @@ int server_reconfigure(conf_t *conf, server_t *server)
 		if ((ret = configure_sockets(conf, server)) != KNOT_EOK) {
 			return ret;
 		}
+
+		if (conf_lmdb_readers(conf) > CONF_MAX_DB_READERS) {
+			log_warning("config, exceeded number of database readers");
+		}
 	}
 
 	/* Reconfigure journal DB. */
