@@ -1,4 +1,4 @@
-/*  Copyright (C) 2020 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2021 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -14,14 +14,16 @@
     along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
+#include "knot/catalog/interpret.h"
 #include "knot/common/log.h"
 #include "knot/dnssec/zone-events.h"
 #include "knot/updates/zone-update.h"
 #include "knot/zone/adds_tree.h"
 #include "knot/zone/adjust.h"
-#include "knot/zone/catalog.h"
+#include "knot/zone/digest.h"
 #include "knot/zone/serial.h"
 #include "knot/zone/zone-diff.h"
+#include "knot/zone/zonefile.h"
 #include "contrib/trim.h"
 #include "contrib/ucw/lists.h"
 
@@ -46,7 +48,7 @@ static int init_incremental(zone_update_t *update, zone_t *zone, zone_contents_t
 	if (update->flags & UPDATE_HYBRID) {
 		update->new_cont = old_contents;
 	} else {
-		ret = apply_prepare_zone_copy(old_contents, &update->new_cont);
+		ret = zone_contents_cow(old_contents, &update->new_cont);
 		if (ret != KNOT_EOK) {
 			changeset_clear(&update->change);
 			return ret;
@@ -159,7 +161,7 @@ int zone_update_init(zone_update_t *update, zone_t *zone, zone_update_flags_t fl
 	return init_base(update, zone, NULL, flags);
 }
 
-int zone_update_from_differences(zone_update_t *update, zone_t *zone, zone_contents_t *old_cont,
+int zone_update_from_differences(zone_update_t *update, conf_t *conf, zone_t *zone, zone_contents_t *old_cont,
 				 zone_contents_t *new_cont, zone_update_flags_t flags, bool ignore_dnssec)
 {
 	if (update == NULL || zone == NULL || new_cont == NULL ||
@@ -199,10 +201,6 @@ int zone_update_from_differences(zone_update_t *update, zone_t *zone, zone_conte
 		return ret;
 	}
 
-	// True if nonempty changes were made but the serial
-	// remained the same and has to be incremented.
-	bool diff_semcheck = (ret == KNOT_ESEMCHECK);
-
 	ret = zone_update_apply_changeset(update, &diff);
 	changeset_clear(&diff);
 	if (ret != KNOT_EOK) {
@@ -210,18 +208,6 @@ int zone_update_from_differences(zone_update_t *update, zone_t *zone, zone_conte
 		return ret;
 	}
 
-	if (diff_semcheck) {
-		ret = zone_update_increment_soa(update, conf());
-		if (ret != KNOT_EOK) {
-			zone_update_clear(update);
-			return ret;
-		}
-		log_zone_info(zone->name, "automatic SOA serial increment");
-		log_zone_notice(zone->name, "automatic SOA increment is deprecated and will "
-		                            "result in error in newer versions, configure "
-		                            "'zonefile-load: difference-no-serial' instead");
-	}
-
 	update->init_cont = new_cont;
 	return KNOT_EOK;
 }
@@ -283,7 +269,7 @@ int zone_update_from_contents(zone_update_t *update, zone_t *zone_without_conten
 	return KNOT_EOK;
 }
 
-int zone_update_start_extra(zone_update_t *update)
+int zone_update_start_extra(zone_update_t *update, conf_t *conf)
 {
 	assert((update->flags & (UPDATE_INCREMENTAL | UPDATE_HYBRID)));
 
@@ -293,7 +279,7 @@ int zone_update_start_extra(zone_update_t *update)
 	}
 
 	if (update->init_cont != NULL) {
-		ret = zone_update_increment_soa(update, conf());
+		ret = zone_update_increment_soa(update, conf);
 		if (ret != KNOT_EOK) {
 			return ret;
 		}
@@ -308,7 +294,7 @@ int zone_update_start_extra(zone_update_t *update)
 			return KNOT_ENOMEM;
 		}
 
-		ret = zone_update_increment_soa(update, conf());
+		ret = zone_update_increment_soa(update, conf);
 		if (ret != KNOT_EOK) {
 			return ret;
 		}
@@ -528,6 +514,9 @@ int zone_update_remove(zone_update_t *update, const knot_rrset_t *rrset)
 	if (update == NULL || rrset == NULL) {
 		return KNOT_EINVAL;
 	}
+	if (knot_rrset_empty(rrset)) {
+		return KNOT_EOK;
+	}
 
 	if ((update->flags & (UPDATE_INCREMENTAL | UPDATE_HYBRID)) && rrset->type != KNOT_RRTYPE_SOA) {
 		int ret = changeset_add_removal(&update->change, rrset, CHANGESET_CHECK);
@@ -645,7 +634,7 @@ static int set_new_soa(zone_update_t *update, unsigned serial_policy)
 	}
 
 	uint32_t old_serial = knot_soa_serial(soa_cpy->rrs.rdata);
-	uint32_t new_serial = serial_next(old_serial, serial_policy);
+	uint32_t new_serial = serial_next(old_serial, serial_policy, 1);
 	if (serial_compare(old_serial, new_serial) != SERIAL_LOWER) {
 		log_zone_warning(update->zone->name, "updated SOA serial is lower "
 		                 "than current, serial %u -> %u",
@@ -733,11 +722,11 @@ static int update_catalog(conf_t *conf, zone_update_t *update)
 	int ret = KNOT_EOK;
 	if ((update->flags & UPDATE_INCREMENTAL)) {
 		ret = catalog_update_from_zone(update->zone->catalog_upd,
-		                               update->change.remove,
+		                               update->change.remove, update->new_cont,
 		                               true, false, update->zone->catalog);
 		if (ret == KNOT_EOK) {
 			ret = catalog_update_from_zone(update->zone->catalog_upd,
-			                               update->change.add,
+			                               update->change.add, update->new_cont,
 			                               false, false, NULL);
 		}
 	} else {
@@ -746,15 +735,14 @@ static int update_catalog(conf_t *conf, zone_update_t *update)
 		                             update->zone->name);
 		if (ret == KNOT_EOK) {
 			ret = catalog_update_from_zone(update->zone->catalog_upd,
-			                               update->zone->contents,
+			                               update->new_cont, update->new_cont,
 			                               false, true, NULL);
 		}
 	}
 
 	if (ret == KNOT_EOK) {
-		log_zone_info(update->zone->name, "catalog reloaded, -%zu +%zu members",
-		              trie_weight(update->zone->catalog_upd->rem),
-		              trie_weight(update->zone->catalog_upd->add));
+		log_zone_info(update->zone->name, "catalog reloaded, %zu updates",
+		              trie_weight(update->zone->catalog_upd->upd));
 		if (kill(getpid(), SIGUSR1) != 0) {
 			ret = knot_map_errno();
 		}
@@ -822,6 +810,54 @@ static void discard_adds_tree(zone_update_t *update)
 	update->new_cont->adds_tree = NULL;
 }
 
+int zone_update_semcheck(zone_update_t *update)
+{
+	if (update == NULL) {
+		return KNOT_EINVAL;
+	}
+
+	zone_tree_t *node_ptrs = (update->flags & UPDATE_INCREMENTAL) ?
+	                         update->a_ctx->node_ptrs : NULL;
+
+	// adjust_cb_nsec3_pointer not needed as we don't check DNSSEC here
+	int ret = zone_adjust_contents(update->new_cont, adjust_cb_flags, NULL,
+	                               false, false, 1, node_ptrs);
+	if (ret != KNOT_EOK) {
+		return ret;
+	}
+
+	sem_handler_t handler = {
+		.cb = err_handler_logger
+	};
+
+	ret = sem_checks_process(update->new_cont, SEMCHECK_MANDATORY_ONLY,
+	                         &handler, time(NULL));
+	if (ret != KNOT_EOK) {
+		// error is logged by the error handler
+		return ret;
+	}
+
+	return KNOT_EOK;
+}
+
+int zone_update_verify_digest(conf_t *conf, zone_update_t *update)
+{
+	conf_val_t val = conf_zone_get(conf, C_ZONEMD_VERIFY, update->zone->name);
+	if (!conf_bool(&val)) {
+		return KNOT_EOK;
+	}
+
+	int ret = zone_contents_digest_verify(update->new_cont);
+	if (ret != KNOT_EOK) {
+		log_zone_error(update->zone->name, "ZONEMD, verification failed (%s)",
+		               knot_strerror(ret));
+	} else {
+		log_zone_info(update->zone->name, "ZONEMD, verification successful");
+	}
+
+	return ret;
+}
+
 int zone_update_commit(conf_t *conf, zone_update_t *update)
 {
 	if (conf == NULL || update == NULL) {
@@ -830,18 +866,12 @@ int zone_update_commit(conf_t *conf, zone_update_t *update)
 
 	int ret = KNOT_EOK;
 
-	if (update->flags & UPDATE_INCREMENTAL) {
-		if (changeset_empty(&update->change) &&
-		    update->zone->contents != NULL) {
-			changeset_clear(&update->change);
-			changeset_clear(&update->extra_ch);
-			zone_update_clear(update);
-			return KNOT_EOK;
-		}
+	if ((update->flags & UPDATE_INCREMENTAL) && changeset_empty(&update->change)) {
+		zone_update_clear(update);
+		return KNOT_EOK;
 	}
-	if (update->flags & UPDATE_INCREMENTAL) {
-		ret = commit_incremental(conf, update);
-	} else if ((update->flags & UPDATE_HYBRID)) {
+
+	if (update->flags & (UPDATE_INCREMENTAL | UPDATE_HYBRID)) {
 		ret = commit_incremental(conf, update);
 	} else {
 		ret = commit_full(conf, update);
@@ -850,6 +880,18 @@ int zone_update_commit(conf_t *conf, zone_update_t *update)
 		return ret;
 	}
 
+	conf_val_t val = conf_zone_get(conf, C_DNSSEC_SIGNING, update->zone->name);
+	bool dnssec = conf_bool(&val);
+	val = conf_zone_get(conf, C_ZONEMD_GENERATE, update->zone->name);
+	unsigned digest_alg = conf_opt(&val);
+	if (digest_alg != ZONE_DIGEST_NONE && !dnssec && // in case of DNSSEC, digest is part of signing routine
+	    ((update->flags & UPDATE_FULL) || zone_update_to(update) != NULL)) {
+		ret = zone_update_add_digest(update, digest_alg, false);
+	}
+	if (ret != KNOT_EOK) {
+		return ret;
+	}
+
 	conf_val_t thr = conf_zone_get(conf, C_ADJUST_THR, update->zone->name);
 	if ((update->flags & (UPDATE_HYBRID | UPDATE_FULL))) {
 		ret = zone_adjust_full(update->new_cont, conf_int(&thr));
@@ -862,10 +904,7 @@ int zone_update_commit(conf_t *conf, zone_update_t *update)
 	}
 
 	/* Check the zone size. */
-	conf_val_t val = conf_zone_get(conf, C_ZONE_MAX_SIZE, update->zone->name);
-	if (val.code != KNOT_EOK) {
-		val = conf_zone_get(conf, C_MAX_ZONE_SIZE, update->zone->name);
-	}
+	val = conf_zone_get(conf, C_ZONE_MAX_SIZE, update->zone->name);
 	size_t size_limit = conf_int(&val);
 
 	if (update->new_cont->size > size_limit) {
@@ -878,7 +917,7 @@ int zone_update_commit(conf_t *conf, zone_update_t *update)
 		bool incr_valid = update->flags & UPDATE_INCREMENTAL;
 		const char *msg_valid = incr_valid ? "incremental " : "";
 
-		ret = knot_dnssec_validate_zone(update, incr_valid);
+		ret = knot_dnssec_validate_zone(update, conf, incr_valid);
 		if (ret != KNOT_EOK) {
 			log_zone_error(update->zone->name, "DNSSEC, %svalidation failed (%s)",
 			               msg_valid, knot_strerror(ret));
@@ -901,8 +940,7 @@ int zone_update_commit(conf_t *conf, zone_update_t *update)
 		return ret;
 	}
 
-	val = conf_zone_get(conf, C_DNSSEC_SIGNING, update->zone->name);
-	if (conf_bool(&val) && zone_is_slave(conf, update->zone)) {
+	if (dnssec && zone_is_slave(conf, update->zone)) {
 		ret = zone_set_lastsigned_serial(update->zone,
 		                                 zone_contents_serial(update->new_cont));
 		if (ret != KNOT_EOK) {
