@@ -1,4 +1,4 @@
-/*  Copyright (C) 2020 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2021 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -18,11 +18,11 @@
 #include <unistd.h>
 #include <urcu.h>
 
+#include "knot/catalog/generate.h"
 #include "knot/common/log.h"
 #include "knot/conf/module.h"
 #include "knot/events/replan.h"
 #include "knot/journal/journal_metadata.h"
-#include "knot/zone/catalog.h"
 #include "knot/zone/timers.h"
 #include "knot/zone/zone-load.h"
 #include "knot/zone/zone.h"
@@ -55,6 +55,14 @@ static bool zone_file_updated(conf_t *conf, const zone_t *old_zone,
 	}
 }
 
+static void zone_get_catalog_group(conf_t *conf, zone_t *zone)
+{
+	conf_val_t val = conf_zone_get(conf, C_CATALOG_GROUP, zone->name);
+	if (val.code == KNOT_EOK) {
+		zone->catalog_group = conf_str(&val);
+	}
+}
+
 static zone_t *create_zone_from(const knot_dname_t *name, server_t *server)
 {
 	zone_t *zone = zone_new(name);
@@ -104,6 +112,12 @@ static zone_t *create_zone_reload(conf_t *conf, const knot_dname_t *name,
 		zone_control_clear(old_zone);
 	}
 
+	zone->cat_members = old_zone->cat_members;
+	old_zone->cat_members = NULL;
+
+	zone->catalog_gen = old_zone->catalog_gen;
+	old_zone->catalog_gen = NULL;
+
 	return zone;
 }
 
@@ -125,6 +139,30 @@ static zone_t *create_zone_new(conf_t *conf, const knot_dname_t *name,
 
 	zone_timers_sanitize(conf, zone);
 
+	conf_val_t role = conf_zone_get(conf, C_CATALOG_ROLE, name);
+	if (conf_opt(&role) == CATALOG_ROLE_MEMBER) {
+		conf_val_t catz = conf_zone_get(conf, C_CATALOG_ZONE, name);
+		assert(catz.code == KNOT_EOK); // conf consistency checked in conf/tools.c
+		zone->catalog_gen = knot_dname_copy(conf_dname(&catz), NULL);
+		if (zone->timers.catalog_member == 0) {
+			zone->timers.catalog_member = time(NULL);
+		}
+		if (zone->catalog_gen == NULL) {
+			log_zone_error(zone->name, "failed to initialize catalog member zone (%s)",
+			               knot_strerror(KNOT_ENOMEM));
+			zone_free(&zone);
+			return NULL;
+		}
+	} else if (conf_opt(&role) == CATALOG_ROLE_GENERATE) {
+		zone->cat_members = catalog_update_new();
+		if (zone->cat_members == NULL) {
+			log_zone_error(zone->name, "failed to initialize catalog zone (%s)",
+			               knot_strerror(KNOT_ENOMEM));
+			zone_free(&zone);
+			return NULL;
+		}
+	}
+
 	if (zone_expired(zone)) {
 		// expired => force bootstrap, no load attempt
 		log_zone_info(zone->name, "zone will be bootstrapped");
@@ -154,11 +192,16 @@ static zone_t *create_zone(conf_t *conf, const knot_dname_t *name, server_t *ser
 	assert(name);
 	assert(server);
 
+	zone_t *z;
+
 	if (old_zone) {
-		return create_zone_reload(conf, name, server, old_zone);
+		z = create_zone_reload(conf, name, server, old_zone);
 	} else {
-		return create_zone_new(conf, name, server);
+		z = create_zone_new(conf, name, server);
 	}
+
+	zone_get_catalog_group(conf, z);
+	return z;
 }
 
 static void mark_changed_zones(knot_zonedb_t *zonedb, trie_t *changed)
@@ -212,7 +255,7 @@ static bool check_open_catalog(catalog_t *cat)
 	if (knot_lmdb_exists(&cat->db)) {
 		int ret = catalog_open(cat);
 		if (ret != KNOT_EOK) {
-			log_error("failed to open existing zone catalog");
+			log_error("failed to open persistent zone catalog");
 		} else {
 			return true;
 		}
@@ -227,15 +270,19 @@ static zone_t *reuse_member_zone(zone_t *zone, server_t *server, conf_t *conf,
 		return NULL;
 	}
 
-	catalog_upd_val_t *upd = catalog_update_get(&server->catalog_upd, zone->name, true);
+	catalog_upd_val_t *upd = catalog_update_get(&server->catalog_upd, zone->name);
 	if (upd != NULL) {
-		if (upd->just_reconf) {
+		switch (upd->type) {
+		case CAT_UPD_UNIQ:
 			zone_purge(conf, zone, server);
 			knot_sem_wait(&zone->cow_lock);
 			ptrlist_add(expired_contents, zone_expire(zone), NULL);
 			knot_sem_post(&zone->cow_lock);
-		} else {
+			break;
+		case CAT_UPD_REM:
 			return NULL; // zone to be removed
+		default:
+			break;
 		}
 	}
 
@@ -253,8 +300,8 @@ static zone_t *reuse_member_zone(zone_t *zone, server_t *server, conf_t *conf,
 // cold start of knot: add unchanged member zone to zonedb
 static zone_t *reuse_cold_zone(const knot_dname_t *zname, server_t *server, conf_t *conf)
 {
-	catalog_upd_val_t *upd = catalog_update_get(&server->catalog_upd, zname, true);
-	if (upd != NULL && !upd->just_reconf) {
+	catalog_upd_val_t *upd = catalog_update_get(&server->catalog_upd, zname);
+	if (upd != NULL && upd->type == CAT_UPD_REM) {
 		return NULL; // zone will be removed immediately
 	}
 
@@ -269,10 +316,29 @@ static zone_t *reuse_cold_zone(const knot_dname_t *zname, server_t *server, conf
 	return zone;
 }
 
+typedef struct {
+	knot_zonedb_t *zonedb;
+	server_t *server;
+	conf_t *conf;
+} reuse_cold_zone_ctx_t;
+
+static int reuse_cold_zone_cb(const knot_dname_t *member, _unused_ const knot_dname_t *owner,
+                              _unused_ const knot_dname_t *catz, _unused_ const char *group,
+                              void *ctx)
+{
+	reuse_cold_zone_ctx_t *rcz = ctx;
+
+	zone_t *zone = reuse_cold_zone(member, rcz->server, rcz->conf);
+	if (zone == NULL) {
+		return KNOT_ENOMEM;
+	}
+	return knot_zonedb_insert(rcz->zonedb, zone);
+}
+
 static zone_t *add_member_zone(catalog_upd_val_t *val, knot_zonedb_t *check,
                                server_t *server, conf_t *conf)
 {
-	if (val->just_reconf) {
+	if (val->type != CAT_UPD_ADD) {
 		return NULL;
 	}
 
@@ -284,7 +350,6 @@ static zone_t *add_member_zone(catalog_upd_val_t *val, knot_zonedb_t *check,
 	zone_t *zone = create_zone(conf, val->member, server, NULL);
 	if (zone == NULL) {
 		log_zone_error(val->member, "zone cannot be created");
-		catalog_del2(conf->catalog, val);
 	} else {
 		zone_set_flag(zone, ZONE_IS_CAT_MEMBER);
 		conf_activate_modules(conf, server, zone->name, &zone->query_modules,
@@ -351,6 +416,12 @@ static knot_zonedb_t *create_zonedb(conf_t *conf, server_t *server, list_t *expi
 		knot_zonedb_insert(db_new, zone);
 	}
 
+	int ret = catalog_update_commit(&server->catalog_upd, &server->catalog);
+	if (ret != KNOT_EOK) {
+		log_error("catalog, failed to apply changes (%s)", knot_strerror(ret));
+		return db_new;
+	}
+
 	if (db_old != NULL) {
 		knot_zonedb_iter_t *it = knot_zonedb_iter_begin(db_old);
 		while (!knot_zonedb_iter_finished(it)) {
@@ -363,59 +434,23 @@ static knot_zonedb_t *create_zonedb(conf_t *conf, server_t *server, list_t *expi
 		}
 		knot_zonedb_iter_free(it);
 	} else if (check_open_catalog(&server->catalog)) {
-		catalog_foreach(&server->catalog) {
-			const knot_dname_t *member = NULL, *catzone = NULL;
-			catalog_curval(&server->catalog, &member, NULL, &catzone);
-
-			if (!conf_rawid_exists(conf, C_ZONE, catzone, knot_dname_size(catzone))) {
-				knot_dname_txt_storage_t cat_str;
-				(void)knot_dname_to_str(cat_str, catzone, sizeof(cat_str));
-				log_zone_error(member, "catalog template zone '%s' not configured, ignoring", cat_str);
-				continue;
-			} else if (conf_rawid_exists(conf, C_ZONE, member, knot_dname_size(member))) {
-				log_zone_error(member, "non-catalog zone already configured, ignoring");
-				continue;
-			}
-
-			zone_t *zone = reuse_cold_zone(member, server, conf);
-			if (zone != NULL) {
-				knot_zonedb_insert(db_new, zone);
-			}
+		reuse_cold_zone_ctx_t rcz = { db_new, server, conf };
+		ret = catalog_apply(&server->catalog, NULL, reuse_cold_zone_cb, &rcz, false);
+		if (ret != KNOT_EOK) {
+			log_error("catalog, failed to reload member zones (%s)", knot_strerror(ret));
 		}
 	}
 
-	catalog_commit_cleanup(&server->catalog);
-
-	catalog_it_t *it = catalog_it_begin(&server->catalog_upd, false);
-	int catret = 1;
-	if (!catalog_it_finished(it)) {
-		catret = catalog_begin(&server->catalog);
-	}
-	while (!catalog_it_finished(it) && catret == KNOT_EOK) {
+	catalog_it_t *it = catalog_it_begin(&server->catalog_upd);
+	while (!catalog_it_finished(it)) {
 		catalog_upd_val_t *val = catalog_it_val(it);
-		if (val->just_reconf || knot_zonedb_find(db_new, val->member) == NULL) {
-			// ^ warning for existing zone later in add_member_zone()
-			catret = catalog_add2(&server->catalog, val);
-		}
-		catalog_it_next(it);
-	}
-	catalog_it_free(it);
-	if (catret == KNOT_EOK) {
-		catret = catalog_commit(&server->catalog);
-	}
-
-	it = catalog_it_begin(&server->catalog_upd, false);
-	while (!catalog_it_finished(it) && catret == KNOT_EOK) {
-		zone_t *zone = add_member_zone(catalog_it_val(it), db_new, server, conf);
+		zone_t *zone = add_member_zone(val, db_new, server, conf);
 		if (zone != NULL) {
 			knot_zonedb_insert(db_new, zone);
 		}
 		catalog_it_next(it);
 	}
 	catalog_it_free(it);
-	if (catret < 0) {
-		log_error("failed to process zone catalog (%s)", knot_strerror(catret));
-	}
 
 	return db_new;
 }
@@ -470,15 +505,10 @@ static void remove_old_zonedb(conf_t *conf, knot_zonedb_t *db_old,
 
 catalog_only:
 	; /* Remove deleted cataloged zones from conf. */
-	catalog_it_t *cat_it = catalog_it_begin(&server->catalog_upd, true);
-	int catret = 1;
-	if (!catalog_it_finished(cat_it)) {
-		catret = catalog_begin(&server->catalog);
-	}
+	catalog_it_t *cat_it = catalog_it_begin(&server->catalog_upd);
 	while (!catalog_it_finished(cat_it)) {
 		catalog_upd_val_t *upd = catalog_it_val(cat_it);
-		if (!upd->just_reconf) {
-			catalog_del(&server->catalog, upd->member);
+		if (upd->type == CAT_UPD_REM) {
 			zone_t *zone = knot_zonedb_find(db_old, upd->member);
 			if (zone != NULL) {
 				zone_purge(conf, zone, server);
@@ -487,12 +517,6 @@ catalog_only:
 		catalog_it_next(cat_it);
 	}
 	catalog_it_free(cat_it);
-	if (catret == KNOT_EOK) {
-		catret = catalog_commit(&server->catalog);
-	}
-	if (catret < 0) {
-		log_error("failed to process zone catalog (%s)", knot_strerror(catret));
-	}
 
 	/* Clear catalog changes. No need to use mutex as this is done from main
 	 * thread while all zone events are paused. */
@@ -514,6 +538,8 @@ void zonedb_reload(conf_t *conf, server_t *server)
 	list_t contents_tofree;
 	init_list(&contents_tofree);
 
+	catalog_update_finalize(&server->catalog_upd, &server->catalog, conf);
+
 	/* Insert all required zones to the new zone DB. */
 	knot_zonedb_t *db_new = create_zonedb(conf, server, &contents_tofree);
 	if (db_new == NULL) {
@@ -521,6 +547,8 @@ void zonedb_reload(conf_t *conf, server_t *server)
 		return;
 	}
 
+	catalogs_generate(db_new, server->zone_db);
+
 	/* Switch the databases. */
 	knot_zonedb_t **db_current = &server->zone_db;
 	knot_zonedb_t *db_old = rcu_xchg_pointer(db_current, db_new);
@@ -533,3 +561,33 @@ void zonedb_reload(conf_t *conf, server_t *server)
 	/* Remove old zone DB. */
 	remove_old_zonedb(conf, db_old, server);
 }
+
+int zone_reload_modules(conf_t *conf, server_t *server, const knot_dname_t *zone_name)
+{
+	zone_t **zone = knot_zonedb_find_ptr(server->zone_db, zone_name);
+	if (zone == NULL) {
+		return KNOT_ENOENT;
+	}
+	assert(knot_dname_is_equal((*zone)->name, zone_name));
+
+	zone_events_freeze_blocking(*zone);
+	knot_sem_wait(&(*zone)->cow_lock);
+
+	zone_t *newzone = create_zone(conf, zone_name, server, *zone);
+	if (newzone == NULL) {
+		return KNOT_ENOMEM;
+	}
+	conf_activate_modules(conf, server, newzone->name, &newzone->query_modules,
+	                      &newzone->query_plan);
+
+	zone_t *oldzone = rcu_xchg_pointer(zone, newzone);
+	synchronize_rcu();
+
+	assert(newzone->contents == oldzone->contents);
+	oldzone->contents = NULL; // contents have been re-used by newzone
+
+	knot_sem_post(&oldzone->cow_lock);
+	zone_free(&oldzone);
+
+	return KNOT_EOK;
+}
